@@ -9,6 +9,7 @@ import time
 import hashlib
 import mimetypes
 import pickle
+import shutil
 from io import BytesIO
 from xml.etree import ElementTree
 
@@ -43,6 +44,10 @@ BACKGROUND_ICONS_SIZE = 100
 BACKGROUND_COLLECTION_TYPE_DIRECTORY = "directory"
 BACKGROUND_COLLECTION_TYPE_XML = "xml"
 
+CONFIG_FOLDER = os.path.join(GLib.get_user_config_dir(), 'cinnamon', 'backgrounds')
+OLD_CONFIG_FOLDER = os.path.expanduser("~/.cinnamon/backgrounds")
+USER_FOLDERS_FILE_NAME = 'user-folders.lst'
+
 # even though pickle supports higher protocol versions, we want to version 2 because it's the latest
 # version supported by python2 which (at this time) is still used by older versions of Cinnamon.
 # When those versions are no longer supported, we can consider using a newer version.
@@ -144,13 +149,13 @@ class ColorsWidget(SettingsWidget):
 
     def on_combo_changed(self, widget, key):
         tree_iter = widget.get_active_iter()
-        if tree_iter != None:
+        if tree_iter is not None:
             value = widget.get_model()[tree_iter][0]
             self.settings.set_string(key, value)
             self.show_or_hide_color2(value)
 
     def show_or_hide_color2(self, value):
-        if (value == 'solid'):
+        if value == 'solid':
             self.color2_button.hide()
         else:
             self.color2_button.show()
@@ -183,7 +188,7 @@ class Module:
 
             self.xdg_pictures_directory = os.path.expanduser("~/Pictures")
             xdg_config = os.path.expanduser("~/.config/user-dirs.dirs")
-            if os.path.exists(xdg_config) and os.path.exists("/usr/bin/xdg-user-dir"):
+            if os.path.exists(xdg_config) and shutil.which("xdg-user-dir"):
                 path = subprocess.check_output(["xdg-user-dir", "PICTURES"]).decode("utf-8").rstrip("\n")
                 if os.path.exists(path):
                     self.xdg_pictures_directory = path
@@ -340,7 +345,9 @@ class Module:
 
     def get_user_backgrounds(self):
         self.user_backgrounds = []
-        path = os.path.expanduser("~/.cinnamon/backgrounds/user-folders.lst")
+        path = os.path.join(CONFIG_FOLDER, USER_FOLDERS_FILE_NAME)
+        old_path = os.path.join(OLD_CONFIG_FOLDER, USER_FOLDERS_FILE_NAME)
+        path = path if os.path.exists(path) else old_path
         if os.path.exists(path):
             with open(path) as f:
                 folders = f.readlines()
@@ -359,7 +366,7 @@ class Module:
 
     def format_source(self, type, path):
         # returns 'type://path'
-        return ("%s://%s" % (type, path))
+        return "%s://%s" % (type, path)
 
     def get_initial_path(self):
         try:
@@ -372,7 +379,7 @@ class Module:
             self.remove_folder_button.set_sensitive(True)
 
             if image_source != "" and "://" in image_source:
-                while tree_iter != None:
+                while tree_iter is not None:
                     if collection_source == image_source:
                         tree_path = self.collection_store.get_path(tree_iter)
                         self.folder_tree.set_cursor(tree_path)
@@ -463,10 +470,10 @@ class Module:
                         break
 
     def update_folder_list(self):
-        path = os.path.expanduser("~/.cinnamon/backgrounds")
+        path = CONFIG_FOLDER
         if not os.path.exists(path):
             os.makedirs(path, mode=0o755, exist_ok=True)
-        path = os.path.expanduser("~/.cinnamon/backgrounds/user-folders.lst")
+        path = os.path.join(CONFIG_FOLDER, USER_FOLDERS_FILE_NAME)
         if len(self.user_backgrounds) == 0:
             file_data = ""
         else:
@@ -621,7 +628,7 @@ class PixCache(object):
                             img = Image.alpha_composite(bg_img, img)
                         img = img.convert("RGB")
                     if size:
-                        img.thumbnail((size, size), Image.ANTIALIAS)
+                        img.thumbnail((size, size), Image.LANCZOS)
 
                     import imtools
                     img = imtools.round_image(img, {}, False, None, 3, 255)
@@ -763,7 +770,7 @@ class ThreadedIconView(Gtk.IconView):
                 if filename.endswith(".xml"):
                     filename = self.getFirstFileFromBackgroundXml(filename)
                 pix = PIX_CACHE.get_pix(filename, BACKGROUND_ICONS_SIZE)
-                if pix != None:
+                if pix is not None:
                     if "name" in to_load:
                         label = to_load["name"]
                     else:
@@ -775,7 +782,7 @@ class ThreadedIconView(Gtk.IconView):
                     dimensions = "%dx%d" % (pix[1], pix[2])
 
                     self._loaded_data_lock.acquire()
-                    self._loaded_data.append((to_load, pix[0], "<b>%s</b>\n<sub>%s%s</sub>" % (label, artist, dimensions), path))
+                    self._loaded_data.append((to_load, pix[0], "<b>%s</b>\n<small>%s%s</small>" % (label, artist, dimensions), path))
                     self._loaded_data_lock.release()
 
         self._loading_lock.acquire()
