@@ -16,7 +16,6 @@ const Tweener = imports.ui.tweener;
 
 const WindowUtils = imports.misc.windowUtils;
 
-const POPUP_APPICON_SIZE = 96;
 const POPUP_SCROLL_TIME = 0.10; // seconds
 const POPUP_DELAY_TIMEOUT = 150; // milliseconds
 const POPUP_FADE_OUT_TIME = 0.1; // seconds
@@ -30,7 +29,7 @@ const THUMBNAIL_FADE_TIME = 0.1; // seconds
 const PREVIEW_DELAY_TIMEOUT = 0; // milliseconds
 var PREVIEW_SWITCHER_FADEOUT_TIME = 0.2; // seconds
 
-const iconSizes = [96, 64, 48, 32, 22];
+const iconSizes = [96, 64, 48];
 
 function mod(a, b) {
     return (a + b) % b;
@@ -76,6 +75,9 @@ ClassicSwitcher.prototype = {
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
         this.actor.connect('allocate', Lang.bind(this, this._allocate));
         
+        this._applist_act_id = 0;
+        this._applist_enter_id = 0;
+
         // Need to force an allocation so we can figure out whether we
         // need to scroll when selecting
         this.actor.opacity = 0;
@@ -122,16 +124,14 @@ ClassicSwitcher.prototype = {
             let thumbnailCenter = posX + icon.width / 2;
             let [childMinWidth, childNaturalWidth] = this._thumbnails.actor.get_preferred_width(-1);
             childBox.x1 = Math.max(monitor.x + leftPadding, Math.floor(thumbnailCenter - childNaturalWidth / 2));
-            if (childBox.x1 + childNaturalWidth > monitor.x + monitor.width - hPadding) {
-                let offset = childBox.x1 + childNaturalWidth - monitor.width + hPadding;
-                childBox.x1 = Math.max(monitor.x + leftPadding, childBox.x1 - offset - hPadding);
+            if (childBox.x1 + childNaturalWidth > monitor.x + monitor.width - rightPadding) {
+                let offset = (childBox.x1 + childNaturalWidth) - (monitor.x + monitor.width - rightPadding);
+                childBox.x1 -= offset;
             }
 
             let spacing = this.actor.get_theme_node().get_length('spacing');
 
             childBox.x2 = childBox.x1 +  childNaturalWidth;
-            if (childBox.x2 > monitor.x + monitor.width - rightPadding)
-                childBox.x2 = monitor.x + monitor.width - rightPadding;
             childBox.y1 = this._appList.actor.allocation.y2 + spacing;
             this._thumbnails.addClones(monitor.y + monitor.height - bottomPadding - childBox.y1);
             let [childMinHeight, childNaturalHeight] = this._thumbnails.actor.get_preferred_height(-1);
@@ -175,6 +175,14 @@ ClassicSwitcher.prototype = {
             return;
         
         if (this._appList) {
+            if (this._applist_act_id !== 0) {
+                this._appList.disconnect(this._applist_act_id);
+                this._applist_act_id = 0;
+            }
+            if (this._applist_enter_id !== 0) {
+                this._appList.disconnect(this._applist_enter_id);
+                this._applist_enter_id = 0;
+            }
             this._clearPreview();
             this._destroyThumbnails();
             this.actor.remove_actor(this._appList.actor);
@@ -185,8 +193,8 @@ ClassicSwitcher.prototype = {
         if (!this._iconsEnabled && !this._thumbnailsEnabled) {
             this._appList.actor.hide();
         }
-        this._appList.connect('item-activated', Lang.bind(this, this._appActivated));
-        this._appList.connect('item-entered', Lang.bind(this, this._appEntered));
+        this._applist_act_id = this._appList.connect('item-activated', Lang.bind(this, this._appActivated));
+        this._applist_enter_id = this._appList.connect('item-entered', Lang.bind(this, this._appEntered));
         
         this._appIcons = this._appList.icons;
         this.actor.get_allocation_box();
@@ -238,6 +246,18 @@ ClassicSwitcher.prototype = {
     },
 
     _onDestroy: function() {
+        if (this._appList !== null) {
+            if (this._applist_act_id > 0) {
+                this._appList.disconnect(this._applist_act_id);
+                this._applist_act_id = 0;
+            }
+
+            if (this._applist_enter_id > 0) {
+                this._appList.disconnect(this._applist_enter_id);
+                this._applist_enter_id = 0;
+            }
+        }
+
         if (this._thumbnailTimeoutId != 0) {
             Mainloop.source_remove(this._thumbnailTimeoutId);
             this._thumbnailTimeoutId = 0;
@@ -402,15 +422,9 @@ AppIcon.prototype = {
             if (window.minimized) {
                 this.label = new St.Label({ text: "[" + title + "]"});               
                 let contrast_effect = new Clutter.BrightnessContrastEffect();                
-                contrast_effect.set_brightness(-0.5, -0.5, -0.5);                         
+                contrast_effect.set_brightness_full(-0.5, -0.5, -0.5);
                 this._iconBin.add_effect(contrast_effect);                
             }
-            else if (window.tile_type == Meta.WindowTileType.TILED) {
-                this.label = new St.Label({ text: "|" + title });
-            }
-            else if (window.tile_type == Meta.WindowTileType.SNAPPED) {
-                this.label = new St.Label({ text: "||" + title });
-            }
             else {
                 this.label = new St.Label({ text: title });    
             }
@@ -428,7 +442,7 @@ AppIcon.prototype = {
     set_size: function(size) {
         if (this.showThumbnail){
             this.icon = new St.Widget();
-            let clones = WindowUtils.createWindowClone(this.window, size, size, true, true);
+            let clones = WindowUtils.createWindowClone(this.window, size * global.ui_scale, size * global.ui_scale, true, true);
             for (let i in clones) {
                 let clone = clones[i];
                 this.icon.add_actor(clone.actor);
@@ -439,11 +453,12 @@ AppIcon.prototype = {
             }
         } else {
             this.icon = this.app ?
-                this.app.create_icon_texture(size) :
+                this.app.create_icon_texture_for_window(size, this.window) :
                 new St.Icon({ icon_name: 'application-default-icon',
                               icon_type: St.IconType.FULLCOLOR,
                               icon_size: size });
         }
+        size *= global.ui_scale;
         this._iconBin.set_size(size, size);
         this._iconBin.child = this.icon;
     }
@@ -462,8 +477,9 @@ SwitcherList.prototype = {
 
         // Here we use a GenericContainer so that we can force all the
         // children except the separator to have the same width.
+        // TODO: Separator is gone, we could use an St.ScrollView now.
         this._list = new Cinnamon.GenericContainer({ style_class: 'switcher-list-item-container' });
-        this._list.spacing = 0;
+        this._list.spacing = -1;
         this._list.connect('style-changed', Lang.bind(this, function() {
                                                         this._list.spacing = this._list.get_theme_node().get_length('spacing');
                                                      }));
@@ -496,7 +512,6 @@ SwitcherList.prototype = {
 
         this._items = [];
         this._highlighted = -1;
-        this._separator = null;
         this._squareItems = squareItems;
         this._minSize = 0;
         this._scrollableRight = true;
@@ -505,6 +520,10 @@ SwitcherList.prototype = {
     },
 
     _allocateTop: function(actor, box, flags) {
+        if (this._list.spacing === -1) {
+            this._list.spacing = this._list.get_theme_node().get_length('spacing');
+        }
+
         let leftPadding = this.actor.get_theme_node().get_padding(St.Side.LEFT);
         let rightPadding = this.actor.get_theme_node().get_padding(St.Side.RIGHT);
 
@@ -570,12 +589,6 @@ SwitcherList.prototype = {
         this._itemEntered(index);
     },
 
-    addSeparator: function () {
-        let box = new St.Bin({ style_class: 'separator' });
-        this._separator = box;
-        this._list.add_actor(box);
-    },
-
     highlight: function(index, justOutline) {
         if (this._highlighted != -1) {
             this._items[this._highlighted].remove_style_pseudo_class('outlined');
@@ -642,36 +655,22 @@ SwitcherList.prototype = {
         this.emit('item-entered', n);
     },
 
-    _maxChildWidth: function (forHeight) {
+    _maxChildWidth: function () {
         let maxChildMin = 0;
         let maxChildNat = 0;
 
-        for (let i = 0; i < this._items.length; i++) {
-            let [childMin, childNat] = this._items[i].get_preferred_width(forHeight);
-            maxChildMin = Math.max(childMin, maxChildMin);
-            maxChildNat = Math.max(childNat, maxChildNat);
-
-            if (this._squareItems) {
-                let [childMin, childNat] = this._items[i].get_preferred_height(-1);
-                maxChildMin = Math.max(childMin, maxChildMin);
-                maxChildNat = Math.max(childNat, maxChildNat);
-            }
+        if (this._items.length > 0) {
+            return this._items[0].get_preferred_width(-1);
         }
 
-        return [maxChildMin, maxChildNat];
+        return [0, 0]
     },
 
     _getPreferredWidth: function (actor, forHeight, alloc) {
-        let [maxChildMin, maxChildNat] = this._maxChildWidth(forHeight);
-
-        let separatorWidth = 0;
-        if (this._separator) {
-            let [sepMin, sepNat] = this._separator.get_preferred_width(forHeight);
-            separatorWidth = sepNat + this._list.spacing;
-        }
+        let [maxChildMin, maxChildNat] = this._maxChildWidth();
 
         let totalSpacing = this._list.spacing * Math.max(1, (this._items.length - 1));
-        alloc.min_size = this._items.length * maxChildMin + separatorWidth + totalSpacing;
+        alloc.min_size = this._items.length * maxChildMin + totalSpacing;
         alloc.natural_size = alloc.min_size;
         this._minSize = alloc.min_size;
     },
@@ -687,7 +686,7 @@ SwitcherList.prototype = {
         }
 
         if (this._squareItems) {
-            let [childMin, childNat] = this._maxChildWidth(-1);
+            let [childMin, childNat] = this._maxChildWidth();
             maxChildMin = Math.max(childMin, maxChildMin);
             maxChildNat = maxChildMin;
         }
@@ -699,17 +698,10 @@ SwitcherList.prototype = {
     _allocate: function (actor, box, flags) {
         let childHeight = box.y2 - box.y1;
 
-        let [maxChildMin, maxChildNat] = this._maxChildWidth(childHeight);
+        let [maxChildMin, maxChildNat] = this._maxChildWidth();
         let totalSpacing = this._list.spacing * (this._items.length - 1);
 
-        let separatorWidth = 0;
-        if (this._separator) {
-            let [sepMin, sepNat] = this._separator.get_preferred_width(childHeight);
-            separatorWidth = sepNat;
-            totalSpacing += this._list.spacing;
-        }
-
-        let childWidth = Math.floor(Math.max(0, box.x2 - box.x1 - totalSpacing - separatorWidth) / this._items.length);
+        let childWidth = Math.floor(Math.max(0, box.x2 - box.x1 - totalSpacing) / this._items.length);
 
         let x = 0;
         let children = this._list.get_children();
@@ -737,14 +729,6 @@ SwitcherList.prototype = {
                 children[i].allocate(childBox, flags);
 
                 x += this._list.spacing + childWidth;
-            } else if (children[i] == this._separator) {
-                // We want the separator to be more compact than the rest.
-                childBox.x1 = x;
-                childBox.y1 = 0;
-                childBox.x2 = x + separatorWidth;
-                childBox.y2 = childHeight;
-                children[i].allocate(childBox, flags);
-                x += this._list.spacing + separatorWidth;
             } else {
                 // Something else, eg, AppList's arrows;
                 // we don't allocate it.
@@ -775,7 +759,7 @@ AppList.prototype = {
         SwitcherList.prototype._init.call(this, true, activeMonitor);
 
         // Construct the AppIcons, add to the popup
-        let activeWorkspace = global.screen.get_active_workspace();
+        let activeWorkspace = global.workspace_manager.get_active_workspace();
         let workspaceIcons = [];
         let otherIcons = [];
         for (let i = 0; i < windows.length; i++) {
@@ -813,8 +797,6 @@ AppList.prototype = {
         let [iconMinHeight, iconNaturalHeight] = this.icons[j].label.get_preferred_height(-1);
         let iconSpacing = iconNaturalHeight + iconPadding + iconBorder;
         let totalSpacing = this._list.spacing * (this._items.length - 1);
-        if (this._separator)
-           totalSpacing += this._separator.width + this._list.spacing;
 
         // We just assume the whole screen here due to weirdness happing with the passed width
         let parentPadding = this.actor.get_parent().get_theme_node().get_horizontal_padding();
@@ -823,15 +805,14 @@ AppList.prototype = {
 
         for(let i =  0; i < iconSizes.length; i++) {
                 this._iconSize = iconSizes[i];
-                height = iconSizes[i] + iconSpacing;
+                height = (iconSizes[i] * global.ui_scale) + iconSpacing;
                 let w = height * this._items.length + totalSpacing;
                 if (w <= availWidth)
                         break;
         }
-
         if (this._items.length == 1) {
             this._iconSize = iconSizes[0];
-            height = iconSizes[0] + iconSpacing;
+            height = (iconSizes[0] * global.ui_scale) + iconSpacing;
         }
 
         for(let i = 0; i < this.icons.length; i++) {
@@ -918,12 +899,7 @@ ThumbnailList.prototype = {
     _init : function(windows, activeMonitor) {
         SwitcherList.prototype._init.call(this, false, activeMonitor);
 
-        let activeWorkspace = global.screen.get_active_workspace();
-
-        // We fake the value of 'separatorAdded' when the app has no window
-        // on the current workspace, to avoid displaying a useless separator in
-        // that case.
-        let separatorAdded = windows.length == 0 || windows[0].get_workspace() != activeWorkspace;
+        let activeWorkspace = global.workspace_manager.get_active_workspace();
 
         this._labels = new Array();
         this._thumbnailBins = new Array();
@@ -931,11 +907,6 @@ ThumbnailList.prototype = {
         this._windows = windows;
 
         for (let i = 0; i < windows.length; i++) {
-            if (!separatorAdded && windows[i].get_workspace() != activeWorkspace) {
-              this.addSeparator();
-              separatorAdded = true;
-            }
-
             let box = new St.BoxLayout({ style_class: 'thumbnail-box',
                                          vertical: true });
 
